@@ -5,6 +5,7 @@ use warnings;
 use 5.020;
 use Archive::Libarchive::Lib;
 use FFI::Platypus::Buffer qw( scalar_to_buffer scalar_to_pointer );
+use FFI::Platypus::Memory qw( strdup free );
 use Ref::Util qw( is_plain_scalarref is_plain_coderef is_blessed_ref is_plain_arrayref );
 use Carp ();
 use PeekPoke::FFI ();
@@ -12,7 +13,7 @@ use experimental qw( signatures );
 use parent qw( Archive::Libarchive::Archive );
 
 # ABSTRACT: Libarchive read archive class
-our $VERSION = '0.02'; # VERSION
+our $VERSION = '0.03'; # VERSION
 
 my $ffi = Archive::Libarchive::Lib->ffi;
 
@@ -27,6 +28,7 @@ $ffi->attach( new => [] => 'opaque' => sub {
 
 $ffi->attach( [ free => 'DESTROY' ] => ['archive_read'] => 'int' => sub {
   my($xsub, $self) = @_;
+  free delete $self->{passphrase} if defined $self->{passphrase};
   return if $self->{cb}                 # inside a callback, we don't own the archive pointer
     || ${^GLOBAL_PHASE} eq 'DESTRUCT';  # during global shutdown the xsub might go away
   my $ret = $xsub->($self);
@@ -159,6 +161,25 @@ $ffi->attach( [data_into_fd => '_read_data_into_fd'] => ['archive_read', 'int']
 $ffi->attach( append_filter => ['archive_read', 'archive_filter_t'] => 'int' );
 $ffi->attach( set_format => ['archive_read', 'archive_format_t'] => 'int' );
 
+
+$ffi->attach( set_passphrase_callback => ['archive_read', 'opaque', 'archive_passphrase_callback'] => 'int' => sub {
+  my($xsub, $self, $sub) = @_;
+
+  my $closure = FFI::Platypus->closure(sub ($r, $) {
+    $r = bless { ptr => $r, cb => 1 }, __PACKAGE__;
+    my $passphrase = $sub->($r);
+    $passphrase = '' unless defined $passphrase;
+    my $ptr = strdup $passphrase;
+    free delete $self->{passphrase} if defined $self->{passphrase};
+    return $self->{passphrase} = $ptr;
+  });
+
+  push @{ $self->{keep} }, $closure;
+
+  $xsub->($self, undef, $closure);
+
+});
+
 require Archive::Libarchive::Lib::ArchiveRead unless $Archive::Libarchive::no_gen;
 
 1;
@@ -175,7 +196,7 @@ Archive::Libarchive::ArchiveRead - Libarchive read archive class
 
 =head1 VERSION
 
-version 0.02
+version 0.03
 
 =head1 SYNOPSIS
 
@@ -347,6 +368,17 @@ Set the format manually.  This will accept either a string representation
 of the format, or the constant.  The constant prefix is C<ARCHIVE_FORMAT_>.
 So for a tar file this would be either C<'tar'> or C<ARCHIVE_FORMAT_TAR>.
 
+=head2 set_passphrase_callback
+
+ # archive_read_set_passphrase_callback
+ my $int = $r->set_passphrase_callback(sub ($r) {
+   ...
+   return $passphrase;
+ });
+
+Set a callback that will be called when a passphrase is required, for example with a .zip
+file with encrypted entries.
+
 =head1 SEE ALSO
 
 =over 4
