@@ -7,11 +7,12 @@ use Archive::Libarchive::Lib;
 use Carp ();
 use Ref::Util qw( is_plain_coderef is_blessed_ref );
 use FFI::Platypus::Buffer qw( window scalar_to_buffer );
+use FFI::Platypus::Memory qw( strdup free );
 use experimental qw( signatures );
 use parent qw( Archive::Libarchive::Archive );
 
 # ABSTRACT: Libarchive write archive class
-our $VERSION = '0.02'; # VERSION
+our $VERSION = '0.03'; # VERSION
 
 my $ffi = Archive::Libarchive::Lib->ffi;
 
@@ -26,6 +27,7 @@ $ffi->attach( new => [] => 'opaque' => sub {
 
 $ffi->attach( [ free => 'DESTROY' ] => ['archive_write'] => 'int' => sub {
   my($xsub, $self) = @_;
+  free delete $self->{passphrase} if defined $self->{passphrase};
   return if $self->{cb}                  # inside a callback, we don't own the archive pointer
     || ${^GLOBAL_PHASE} eq 'DESTRUCT';   # during global shutdown, the xsub might go away
   my $ret = $xsub->($self);
@@ -133,6 +135,27 @@ $ffi->attach( [data_block => '_write_data_block'] => ['archive_write', 'opaque',
 $ffi->attach( add_filter => ['archive_write', 'archive_filter_t'] => 'int' );
 $ffi->attach( set_format => ['archive_write', 'archive_format_t'] => 'int' );
 
+
+$ffi->attach( set_passphrase_callback => ['archive_write', 'opaque', 'archive_passphrase_callback'] => 'int' => sub {
+  my($xsub, $self, $sub) = @_;
+
+  my $closure = FFI::Platypus->closure(sub ($w, $) {
+    $w = bless { ptr => $w, cb => 1 }, __PACKAGE__;
+    my $passphrase = $sub->($w);
+    $passphrase = '' unless defined $passphrase;
+    my $ptr = strdup $passphrase;
+    free delete $self->{passphrase} if defined $self->{passphrase};
+    return $self->{passphrase} = $ptr;
+  });
+
+  push @{ $self->{keep} }, $closure;
+
+  $xsub->($self, undef, $closure);
+
+});
+
+
+
 require Archive::Libarchive::Lib::ArchiveWrite unless $Archive::Libarchive::no_gen;
 
 1;
@@ -149,7 +172,7 @@ Archive::Libarchive::ArchiveWrite - Libarchive write archive class
 
 =head1 VERSION
 
-version 0.02
+version 0.03
 
 =head1 SYNOPSIS
 
@@ -291,6 +314,17 @@ Set the output format.  This will accept either a string representation
 of the format, or the constant.  The constant prefix is C<ARCHIVE_FORMAT_>.
 So for a tar file this would be either C<'tar'> or C<ARCHIVE_FORMAT_TAR>.
 
+=head2 set_passphrase_callback
+
+ # archive_write_set_passphrase_callback
+ my $int = $w->set_passphrase_callback(sub ($w) {
+   ...
+   return $passphrase;
+ });
+
+Set a callback that will be called when a passphrase is required, for example with a .zip
+file with encrypted entries.
+
 =head1 SEE ALSO
 
 =over 4
